@@ -42,9 +42,10 @@
 #include "util.h"
 #include "message.h"
 #include "DlAbortEx.h"
-#include "MessageDigestHelper.h"
+#include "message_digest_helper.h"
 #include "DiskAdaptor.h"
 #include "Logger.h"
+#include "LogFactory.h"
 #include "Peer.h"
 #include "Piece.h"
 #include "PieceStorage.h"
@@ -52,35 +53,34 @@
 #include "BtMessageFactory.h"
 #include "BtRequestFactory.h"
 #include "PeerConnection.h"
-#include "StringFormat.h"
+#include "fmt.h"
 #include "DownloadContext.h"
+#include "PeerStorage.h"
 
 namespace aria2 {
 
 const std::string BtPieceMessage::NAME("piece");
 
 BtPieceMessage::BtPieceMessage
-(size_t index, uint32_t begin, size_t blockLength):
-  AbstractBtMessage(ID, NAME),
-  index_(index),
-  begin_(begin),
-  blockLength_(blockLength),
-  block_(0),
-  rawData_(0)
+(size_t index, int32_t begin, int32_t blockLength)
+  : AbstractBtMessage(ID, NAME),
+    index_(index),
+    begin_(begin),
+    blockLength_(blockLength),
+    data_(0)
 {
   setUploading(true);
 }
 
 BtPieceMessage::~BtPieceMessage()
 {
-  delete [] rawData_;
+  delete [] data_;
 }
 
 void BtPieceMessage::setRawMessage(unsigned char* data)
 {
-  delete [] rawData_;
-  rawData_ = data;
-  block_ = data+9;
+  delete [] data_;
+  data_ = data;
 }
 
 BtPieceMessageHandle BtPieceMessage::create
@@ -107,38 +107,39 @@ void BtPieceMessage::doReceivedAction()
     getPeer()->snubbing(false);
     SharedHandle<Piece> piece = getPieceStorage()->getPiece(index_);
     off_t offset = (off_t)index_*downloadContext_->getPieceLength()+begin_;
-    if(getLogger()->debug()) {
-      getLogger()->debug(MSG_PIECE_RECEIVED,
-                         util::itos(getCuid()).c_str(),
-                         static_cast<unsigned long>(index_),
-                         begin_, blockLength_,
-                         static_cast<long long int>(offset),
-                         static_cast<unsigned long>(slot.getBlockIndex()));
+    A2_LOG_DEBUG(fmt(MSG_PIECE_RECEIVED,
+                     getCuid(),
+                     static_cast<unsigned long>(index_),
+                     begin_,
+                     blockLength_,
+                     static_cast<long long int>(offset),
+                     static_cast<unsigned long>(slot.getBlockIndex())));
+    if(piece->hasBlock(slot.getBlockIndex())) {
+      A2_LOG_DEBUG("Already have this block.");
+      return;
     }
     getPieceStorage()->getDiskAdaptor()->writeData
-      (block_, blockLength_, offset);
+      (data_+9, blockLength_, offset);
     piece->completeBlock(slot.getBlockIndex());
-    if(getLogger()->debug()) {
-      getLogger()->debug(MSG_PIECE_BITFIELD, util::itos(getCuid()).c_str(),
-                         util::toHex(piece->getBitfield(),
-                                     piece->getBitfieldLength()).c_str());
-    }
-    piece->updateHash(begin_, block_, blockLength_);
+    A2_LOG_DEBUG(fmt(MSG_PIECE_BITFIELD, getCuid(),
+                     util::toHex(piece->getBitfield(),
+                                 piece->getBitfieldLength()).c_str()));
+    piece->updateHash(begin_, data_+9, blockLength_);
     getBtMessageDispatcher()->removeOutstandingRequest(slot);
     if(piece->pieceComplete()) {
       if(checkPieceHash(piece)) {
         onNewPiece(piece);
       } else {
         onWrongPiece(piece);
+        peerStorage_->addBadPeer(getPeer()->getIPAddress());
         throw DL_ABORT_EX("Bad piece hash.");
       }
     }
   } else {
-    if(getLogger()->debug()) {
-      getLogger()->debug("CUID#%s - RequestSlot not found, index=%lu, begin=%u",
-                         util::itos(getCuid()).c_str(),
-                         static_cast<unsigned long>(index_), begin_);
-    }
+    A2_LOG_DEBUG(fmt("CUID#%lld - RequestSlot not found, index=%lu, begin=%d",
+                     getCuid(),
+                     static_cast<unsigned long>(index_),
+                     begin_));
   }
 }
 
@@ -173,32 +174,26 @@ void BtPieceMessage::send()
   }
   size_t writtenLength;
   if(!isSendingInProgress()) {
-    if(getLogger()->info()) {
-      getLogger()->info(MSG_SEND_PEER_MESSAGE,
-                        util::itos(getCuid()).c_str(),
-                        getPeer()->getIPAddress().c_str(),
-                        getPeer()->getPort(),
-                        toString().c_str());
-    }
+    A2_LOG_INFO(fmt(MSG_SEND_PEER_MESSAGE,
+                    getCuid(),
+                    getPeer()->getIPAddress().c_str(),
+                    getPeer()->getPort(),
+                    toString().c_str()));
     unsigned char* msgHdr = createMessageHeader();
     size_t msgHdrLen = getMessageHeaderLength();
-    if(getLogger()->debug()) {
-      getLogger()->debug("msglength = %lu bytes",
-                         static_cast<unsigned long>(msgHdrLen+blockLength_));
-    }
+    A2_LOG_DEBUG(fmt("msglength = %lu bytes",
+                     static_cast<unsigned long>(msgHdrLen+blockLength_)));
     getPeerConnection()->pushBytes(msgHdr, msgHdrLen);
-    getPeerConnection()->sendPendingData();
     off_t pieceDataOffset =
       (off_t)index_*downloadContext_->getPieceLength()+begin_;
-    writtenLength = sendPieceData(pieceDataOffset, blockLength_);
-  } else {
-    writtenLength = getPeerConnection()->sendPendingData();
+    pushPieceData(pieceDataOffset, blockLength_);
   }
+  writtenLength = getPeerConnection()->sendPendingData();
   getPeer()->updateUploadLength(writtenLength);
   setSendingInProgress(!getPeerConnection()->sendBufferIsEmpty());
 }
 
-size_t BtPieceMessage::sendPieceData(off_t offset, size_t length) const
+void BtPieceMessage::pushPieceData(off_t offset, int32_t length) const
 {
   assert(length <= 16*1024);
   unsigned char* buf = new unsigned char[length];
@@ -209,9 +204,8 @@ size_t BtPieceMessage::sendPieceData(off_t offset, size_t length) const
     delete [] buf;
     throw;
   }
-  if(r == static_cast<ssize_t>(length)) {
+  if(r == length) {
     getPeerConnection()->pushBytes(buf, length);
-    return getPeerConnection()->sendPendingData();
   } else {
     throw DL_ABORT_EX(EX_DATA_READ);
   }
@@ -219,23 +213,22 @@ size_t BtPieceMessage::sendPieceData(off_t offset, size_t length) const
 
 std::string BtPieceMessage::toString() const
 {
-  return strconcat(NAME, " index=", util::itos(index_), ", begin=",
-                   util::itos(begin_), ", length=", util::itos(blockLength_));
+  return fmt("%s index=%lu, begin=%d, length=%d",
+             NAME.c_str(),
+             static_cast<unsigned long>(index_),
+             begin_, blockLength_);
 }
 
 bool BtPieceMessage::checkPieceHash(const SharedHandle<Piece>& piece)
 {
   if(!getPieceStorage()->isEndGame() && piece->isHashCalculated()) {
-    if(getLogger()->debug()) {
-      getLogger()->debug("Hash is available!! index=%lu",
-                         static_cast<unsigned long>(piece->getIndex()));
-    }
+    A2_LOG_DEBUG(fmt("Hash is available!! index=%lu",
+                     static_cast<unsigned long>(piece->getIndex())));
     return
-      piece->getHashString()==downloadContext_->getPieceHash(piece->getIndex());
+      piece->getDigest() == downloadContext_->getPieceHash(piece->getIndex());
   } else {
     off_t offset = (off_t)piece->getIndex()*downloadContext_->getPieceLength();
-    
-    return MessageDigestHelper::staticSHA1Digest
+    return message_digest::staticSHA1Digest
       (getPieceStorage()->getDiskAdaptor(), offset, piece->getLength())
       == downloadContext_->getPieceHash(piece->getIndex());
   }
@@ -243,22 +236,18 @@ bool BtPieceMessage::checkPieceHash(const SharedHandle<Piece>& piece)
 
 void BtPieceMessage::onNewPiece(const SharedHandle<Piece>& piece)
 {
-  if(getLogger()->info()) {
-    getLogger()->info(MSG_GOT_NEW_PIECE,
-                      util::itos(getCuid()).c_str(),
-                      static_cast<unsigned long>(piece->getIndex()));
-  }
+  A2_LOG_INFO(fmt(MSG_GOT_NEW_PIECE,
+                  getCuid(),
+                  static_cast<unsigned long>(piece->getIndex())));
   getPieceStorage()->completePiece(piece);
   getPieceStorage()->advertisePiece(getCuid(), piece->getIndex());
 }
 
 void BtPieceMessage::onWrongPiece(const SharedHandle<Piece>& piece)
 {
-  if(getLogger()->info()) {
-    getLogger()->info(MSG_GOT_WRONG_PIECE,
-                      util::itos(getCuid()).c_str(),
-                      static_cast<unsigned long>(piece->getIndex()));
-  }
+  A2_LOG_INFO(fmt(MSG_GOT_WRONG_PIECE,
+                  getCuid(),
+                  static_cast<unsigned long>(piece->getIndex())));
   erasePieceOnDisk(piece);
   piece->clearAllBlock();
   piece->destroyHashContext();
@@ -286,12 +275,11 @@ void BtPieceMessage::onChokingEvent(const BtChokingEvent& event)
   if(!isInvalidate() &&
      !isSendingInProgress() &&
      !getPeer()->isInAmAllowedIndexSet(index_)) {
-    if(getLogger()->debug()) {
-      getLogger()->debug(MSG_REJECT_PIECE_CHOKED,
-                         util::itos(getCuid()).c_str(),
-                         static_cast<unsigned long>(index_),
-                         begin_, blockLength_);
-    }
+    A2_LOG_DEBUG(fmt(MSG_REJECT_PIECE_CHOKED,
+                     getCuid(),
+                     static_cast<unsigned long>(index_),
+                     begin_,
+                     blockLength_));
     if(getPeer()->isFastExtensionEnabled()) {
       BtMessageHandle rej =
         getBtMessageFactory()->createRejectMessage
@@ -310,12 +298,11 @@ void BtPieceMessage::onCancelSendingPieceEvent
      index_ == event.getIndex() &&
      begin_ == event.getBegin() &&
      blockLength_ == event.getLength()) {
-    if(getLogger()->debug()) {
-      getLogger()->debug(MSG_REJECT_PIECE_CANCEL,
-                         util::itos(getCuid()).c_str(),
-                         static_cast<unsigned long>(index_),
-                         begin_, blockLength_);
-    }
+    A2_LOG_DEBUG(fmt(MSG_REJECT_PIECE_CANCEL,
+                     getCuid(),
+                     static_cast<unsigned long>(index_),
+                     begin_,
+                     blockLength_));
     if(getPeer()->isFastExtensionEnabled()) {
       BtMessageHandle rej =
         getBtMessageFactory()->createRejectMessage
@@ -332,4 +319,10 @@ void BtPieceMessage::setDownloadContext
   downloadContext_ = downloadContext;
 }
 
+void BtPieceMessage::setPeerStorage
+(const SharedHandle<PeerStorage>& peerStorage)
+{
+  peerStorage_ = peerStorage;
+}
+
 } // namespace aria2
