@@ -47,26 +47,26 @@
 #include "bittorrent_helper.h"
 #include "bitfield.h"
 #include "wallclock.h"
+#include "fmt.h"
 
 namespace aria2 {
 
-DHTBucket::DHTBucket(size_t prefixLength,
-                     const unsigned char* max, const unsigned char* min,
-                     const SharedHandle<DHTNode>& localNode):
-  prefixLength_(prefixLength),
-  localNode_(localNode),
-  lastUpdated_(global::wallclock),
-  logger_(LogFactory::getInstance())
+DHTBucket::DHTBucket
+(size_t prefixLength,
+ const unsigned char* max, const unsigned char* min,
+ const SharedHandle<DHTNode>& localNode)
+  : prefixLength_(prefixLength),
+    localNode_(localNode),
+    lastUpdated_(global::wallclock())
 {
   memcpy(max_, max, DHT_ID_LENGTH);
   memcpy(min_, min, DHT_ID_LENGTH);
 }
 
-DHTBucket::DHTBucket(const SharedHandle<DHTNode>& localNode):
-  prefixLength_(0),
-  localNode_(localNode),
-  lastUpdated_(global::wallclock),
-  logger_(LogFactory::getInstance())
+DHTBucket::DHTBucket(const SharedHandle<DHTNode>& localNode)
+  : prefixLength_(0),
+    localNode_(localNode),
+    lastUpdated_(global::wallclock())
 {
   memset(max_, 0xffu, DHT_ID_LENGTH);
   memset(min_, 0, DHT_ID_LENGTH);
@@ -111,7 +111,7 @@ bool DHTBucket::addNode(const SharedHandle<DHTNode>& node)
 {
   notifyUpdate();
   std::deque<SharedHandle<DHTNode> >::iterator itr =
-    std::find(nodes_.begin(), nodes_.end(), node);
+    std::find_if(nodes_.begin(), nodes_.end(), derefEqual(node));
   if(itr == nodes_.end()) {
     if(nodes_.size() < K) {
       nodes_.push_back(node);
@@ -145,7 +145,7 @@ void DHTBucket::dropNode(const SharedHandle<DHTNode>& node)
 {
   if(cachedNodes_.size()) {
     std::deque<SharedHandle<DHTNode> >::iterator itr =
-      find(nodes_.begin(), nodes_.end(), node);
+      std::find_if(nodes_.begin(), nodes_.end(), derefEqual(node));
     if(itr != nodes_.end()) {
       nodes_.erase(itr);
       nodes_.push_back(cachedNodes_.front());
@@ -157,7 +157,7 @@ void DHTBucket::dropNode(const SharedHandle<DHTNode>& node)
 void DHTBucket::moveToHead(const SharedHandle<DHTNode>& node)
 {
   std::deque<SharedHandle<DHTNode> >::iterator itr =
-    std::find(nodes_.begin(), nodes_.end(), node);
+    std::find_if(nodes_.begin(), nodes_.end(), derefEqual(node));
   if(itr != nodes_.end()) {
     nodes_.erase(itr);
     nodes_.push_front(node);
@@ -167,7 +167,7 @@ void DHTBucket::moveToHead(const SharedHandle<DHTNode>& node)
 void DHTBucket::moveToTail(const SharedHandle<DHTNode>& node)
 {
   std::deque<SharedHandle<DHTNode> >::iterator itr =
-    std::find(nodes_.begin(), nodes_.end(), node);
+    std::find_if(nodes_.begin(), nodes_.end(), derefEqual(node));
   if(itr != nodes_.end()) {
     nodes_.erase(itr);
     nodes_.push_back(node);
@@ -206,16 +206,14 @@ SharedHandle<DHTBucket> DHTBucket::split()
   }
   nodes_ = lNodes;
   // TODO create toString() and use it.
-  if(logger_->debug()) {
-    logger_->debug("New bucket. prefixLength=%u, Range:%s-%s",
+  A2_LOG_DEBUG(fmt("New bucket. prefixLength=%u, Range:%s-%s",
                    static_cast<unsigned int>(rBucket->getPrefixLength()),
                    util::toHex(rBucket->getMinID(), DHT_ID_LENGTH).c_str(),
-                   util::toHex(rBucket->getMaxID(), DHT_ID_LENGTH).c_str());
-    logger_->debug("Existing bucket. prefixLength=%u, Range:%s-%s",
+                   util::toHex(rBucket->getMaxID(), DHT_ID_LENGTH).c_str()));
+  A2_LOG_DEBUG(fmt("Existing bucket. prefixLength=%u, Range:%s-%s",
                    static_cast<unsigned int>(prefixLength_),
                    util::toHex(getMinID(), DHT_ID_LENGTH).c_str(),
-                   util::toHex(getMaxID(), DHT_ID_LENGTH).c_str());
-  }
+                   util::toHex(getMaxID(), DHT_ID_LENGTH).c_str()));
   return rBucket;
 }
 
@@ -233,8 +231,9 @@ SharedHandle<DHTNode> DHTBucket::getNode(const unsigned char* nodeID, const std:
   node->setIPAddress(ipaddr);
   node->setPort(port);
   std::deque<SharedHandle<DHTNode> >::const_iterator itr =
-    std::find(nodes_.begin(), nodes_.end(), node);
-  if(itr == nodes_.end()) {
+    std::find_if(nodes_.begin(), nodes_.end(), derefEqual(node));
+  if(itr == nodes_.end() ||
+     (*itr)->getIPAddress() != ipaddr || (*itr)->getPort() != port) {
     return SharedHandle<DHTNode>();
   } else {
     return *itr;
@@ -250,12 +249,12 @@ bool DHTBucket::operator==(const DHTBucket& bucket) const
 bool DHTBucket::needsRefresh() const
 {
   return nodes_.size() < K ||
-    lastUpdated_.difference(global::wallclock) >= DHT_BUCKET_REFRESH_INTERVAL;
+    lastUpdated_.difference(global::wallclock()) >= DHT_BUCKET_REFRESH_INTERVAL;
 }
 
 void DHTBucket::notifyUpdate()
 {
-  lastUpdated_ = global::wallclock;
+  lastUpdated_ = global::wallclock();
 }
 
 namespace {
