@@ -34,7 +34,7 @@
 /* copyright --> */
 #include "SessionSerializer.h"
 
-#include <fstream>
+#include <cstdio>
 #include <iterator>
 
 #include "RequestGroupMan.h"
@@ -48,6 +48,9 @@
 #include "prefs.h"
 #include "util.h"
 #include "array_fun.h"
+#include "BufferedFile.h"
+#include "OptionParser.h"
+#include "OptionHandler.h"
 
 namespace aria2 {
 
@@ -60,127 +63,115 @@ SessionSerializer::SessionSerializer
 
 bool SessionSerializer::save(const std::string& filename) const
 {
-  std::string tempFilename = strconcat(filename, "__temp");
+  std::string tempFilename = filename;
+  tempFilename += "__temp";
   {
-    std::ofstream out(tempFilename.c_str(), std::ios::binary);
-    if(!out) {
+    BufferedFile fp(tempFilename, BufferedFile::WRITE);
+    if(!fp) {
       return false;
     }
-    save(out);
-    out.flush();
-    if(!out) {
+    if(!save(fp) || fp.close() == EOF) {
       return false;
     }
   }
   return File(tempFilename).renameTo(filename);
 }
 
-
 namespace {
-const std::vector<std::string>& getCumulativeOpts()
+bool writeOption(BufferedFile& fp, const SharedHandle<Option>& op)
 {
-  static std::string cumulativeOpts[] = { PREF_INDEX_OUT, PREF_HEADER };
-  static std::vector<std::string> opts
-    (vbegin(cumulativeOpts), vend(cumulativeOpts));
-  return opts;
-}
-} // namespace
-
-namespace {
-bool inCumulativeOpts(const std::string& opt)
-{
-  const std::vector<std::string>& cumopts = getCumulativeOpts();
-  for(std::vector<std::string>::const_iterator itr = cumopts.begin(),
-        eoi = cumopts.end(); itr != eoi; ++itr) {
-    if(opt == *itr) {
-      return true;
-    }
-  }
-  return false;
-}
-} // namespace
-
-namespace {
-void writeOption(std::ostream& out, const SharedHandle<Option>& op)
-{
-  const std::set<std::string>& requestOptions = listRequestOptions();
-  for(std::set<std::string>::const_iterator itr = requestOptions.begin(),
-        eoi = requestOptions.end(); itr != eoi; ++itr) {
-    if(inCumulativeOpts(*itr)) {
-      continue;
-    }
-    if(op->defined(*itr)) {
-      out << " " << *itr << "=" << op->get(*itr) << "\n";
-    }
-  }
-  const std::vector<std::string>& cumopts = getCumulativeOpts();
-  for(std::vector<std::string>::const_iterator opitr = cumopts.begin(),
-        eoi = cumopts.end(); opitr != eoi; ++opitr) {
-    if(op->defined(*opitr)) {
-      std::vector<std::string> v;
-      util::split(op->get(*opitr), std::back_inserter(v), "\n",
-                  false, false);
-      for(std::vector<std::string>::const_iterator i = v.begin(), eoi = v.end();
-          i != eoi; ++i) {
-        out << " " << *opitr << "=" << *i << "\n";
+  const SharedHandle<OptionParser>& oparser = OptionParser::getInstance();
+  for(size_t i = 1, len = option::countOption(); i < len; ++i) {
+    const Pref* pref = option::i2p(i);
+    const SharedHandle<OptionHandler>& h = oparser->find(pref);
+    if(h && h->getInitialOption() && op->defined(pref)) {
+      if(h->getCumulative()) {
+        const std::string& val = op->get(pref);
+        std::vector<std::string> v;
+        util::split(val.begin(), val.end(), std::back_inserter(v), '\n',
+                    false, false);
+        for(std::vector<std::string>::const_iterator j = v.begin(),
+              eoj = v.end(); j != eoj; ++j) {
+          if(fp.printf(" %s=%s\n", pref->k, (*j).c_str()) < 0) {
+            return false;
+          }
+        }
+      } else {
+        if(fp.printf(" %s=%s\n", pref->k, op->get(pref).c_str()) < 0) {
+          return false;
+        }
       }
     }
   }
+  return true;
 }
 } // namespace
 
 namespace {
-void writeDownloadResult
-(std::ostream& out, std::set<int64_t>& metainfoCache,
+bool writeDownloadResult
+(BufferedFile& fp, std::set<int64_t>& metainfoCache,
  const SharedHandle<DownloadResult>& dr)
 {
   const SharedHandle<MetadataInfo>& mi = dr->metadataInfo;
   if(dr->belongsTo != 0 || (mi && mi->dataOnly())) {
-    return;
+    return true;
   }
   if(!mi) {
     // only save first file entry
     if(dr->fileEntries.empty()) {
-      return;
+      return true;
     }
     const SharedHandle<FileEntry>& file = dr->fileEntries[0];
     std::vector<std::string> uris;
     file->getUris(uris);
     if(uris.empty()) {
-      return;
+      return true;
+    }
+    for(std::vector<std::string>::const_iterator i = uris.begin(),
+          eoi = uris.end(); i != eoi; ++i) {
+      if(fp.printf("%s\t", (*i).c_str()) < 0) {
+        return false;
+      }
+    }
+    if(fp.write("\n", 1) != 1) {
+      return false;
     }
-    std::copy(uris.begin(), uris.end(),
-              std::ostream_iterator<std::string>(out, "\t"));
-    out << "\n";
   } else {
     if(metainfoCache.count(mi->getId()) != 0) {
-      return;
+      return true;
     } else {
       metainfoCache.insert(mi->getId());
-      out << mi->getUri() << "\n";
+      if(fp.printf("%s\n", mi->getUri().c_str()) < 0) {
+        return false;
+      }
     }
   }
-  writeOption(out, dr->option);
+  return writeOption(fp, dr->option);
 }
 } // namespace
 
-void SessionSerializer::save(std::ostream& out) const
+bool SessionSerializer::save(BufferedFile& fp) const
 {
   std::set<int64_t> metainfoCache;
   const std::deque<SharedHandle<DownloadResult> >& results =
     rgman_->getDownloadResults();
   for(std::deque<SharedHandle<DownloadResult> >::const_iterator itr =
         results.begin(), eoi = results.end(); itr != eoi; ++itr) {
-    if((*itr)->result == error_code::FINISHED) {
+    if((*itr)->result == error_code::FINISHED ||
+       (*itr)->result == error_code::REMOVED) {
       continue;
     } else if((*itr)->result == error_code::IN_PROGRESS) {
       if(saveInProgress_) {
-        writeDownloadResult(out, metainfoCache, *itr);
+        if(!writeDownloadResult(fp, metainfoCache, *itr)) {
+          return false;
+        }
       }
     } else {
       // error download
       if(saveError_) {
-        writeDownloadResult(out, metainfoCache, *itr);
+        if(!writeDownloadResult(fp, metainfoCache, *itr)) {
+          return false;
+        }
       }
     }
   }
@@ -190,11 +181,19 @@ void SessionSerializer::save(std::ostream& out) const
     for(std::deque<SharedHandle<RequestGroup> >::const_iterator itr =
           groups.begin(), eoi = groups.end(); itr != eoi; ++itr) {
       SharedHandle<DownloadResult> result = (*itr)->createDownloadResult();
-      writeDownloadResult(out, metainfoCache, result);
+      if(!writeDownloadResult(fp, metainfoCache, result)) {
+        return false;
+      }
+      // PREF_PAUSE was removed from option, so save it here looking
+      // property separately.
+      if((*itr)->isPauseRequested()) {
+        if(fp.printf(" %s=true\n", PREF_PAUSE->k) < 0) {
+          return false;
+        }
+      }
     }
   }
+  return true;
 }
 
 } // namespace aria2
-
-
